# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# groupvte - settings
# Copyright (C) 2005 Michele Baldessari <michele@pupazzo.org>

# This file may be distributed and/or modified under the terms of
# the GNU General Public License version 2 as published by
# the Free Software Foundation.
# This file is distributed without any warranty; without even the implied
# warranty of merchantability or fitness for a particular purpose.
# See "LICENSE.GPL" in the source distribution for more information.

import vte
import gtk
import gtk.gdk
import gconf
import logging

_dir_separator = "/"

class Settings:
    _notebookview = "notebook"
    _tableview    = "table"

    def get_session_list(self):
        s = self.client.all_dirs ("/apps/groupvte/sessions")
        t = []
        for i in s:
            t.append(i.split(_dir_separator)[-1])
        return t

    def get_session_hash(self):
        s = self.client.all_dirs ("/apps/groupvte/sessions")
        t = {}
        for i in s:
            m = self.client.get_list (i + _dir_separator + "machines", gconf.VALUE_STRING)
            if len(m) == 0:
                m = ["Default"]

            t[i.split(_dir_separator)[-1]] = m
        return t

    def has_session(self, session):
        s = "/apps/groupvte/sessions/%s" % (session)
        if self.client.dir_exists(s):
            return True
        else:
            return False

    def has_machine(self, machine):
        s = "/apps/groupvte/machines/%s" % (machine)
        if self.client.dir_exists(s):
            return True
        else:
            return False
    
    def has_session(self, session):
        s = "/apps/groupvte/sessions/%s" % (session)
        if self.client.dir_exists(s):
            return True
        else:
            return False

    def get_machine_list(self):
        s = self.client.all_dirs ("/apps/groupvte/machines")
        t = []
        for i in s:
            t.append(i.split(_dir_separator)[-1])
        return t

    def get_machine_hash(self):
        s = self.client.all_dirs ("/apps/groupvte/machines")
        t = {}
        for i in s:
            m = self.client.get_list (i + _dir_separator + "commands", gconf.VALUE_STRING)
            t[i.split(_dir_separator)[-1]] = m
        return t

    def get_machine(self, machine):
        s = "/apps/groupvte/machines/%s/commands" % (machine)
        return self.client.get_list(s, gconf.VALUE_STRING)

    def get_session(self, session):
        s = "/apps/groupvte/sessions/%s/machines" % (session)
        return self.client.get_list(s, gconf.VALUE_STRING)

    def get_font(self):
        s = self.client.get_string ("/apps/groupvte/global/font")
        return s

    def get_bell(self):
        s = self.client.get_string ("/apps/groupvte/global/bell")
        return s
    
    def get_default_session(self):
        s = self.client.get_string ("/apps/groupvte/global/default_session")
        return s
    
    def get_view(self):
        s = self.client.get_string ("/apps/groupvte/global/view")
        return s

    def get_command(self, machine):
        s = "/apps/groupvte/machines/%s/commands" % (machine)
        return self.client.get_list(s, gconf.VALUE_STRING)

    def get_default_command(self):
        s = "/apps/groupvte/machines/%s/commands" % (self.get_default_session())
        return self.client.get_list(s, gconf.VALUE_STRING)

    def set_view(self, view):
        if view == self._tableview or view == self._notebookview:
            self.client.set_string ("/apps/groupvte/global/view", view)
    
    def set_font(self, font):
        self.client.set_string ("/apps/groupvte/global/font", font)

    def set_bell(self, bell):
        self.client.set_bool ("/apps/groupvte/global/bell", bell)
    
    def set_default_session(self, session):
        self.client.set_string ("/apps/groupvte/global/default_session", session)

    def add_machine(self, machine, commands):
        machine_dir = "/apps/groupvte/machines/%s" % (machine) 
        if self.client.dir_exists(machine_dir):
            return False

        self.client.add_dir(machine_dir, gconf.CLIENT_PRELOAD_ONELEVEL)
        self.client.set_list(machine_dir + "/commands", gconf.VALUE_STRING, commands)
        self.client.suggest_sync()
        return True

    def change_machine(self, machine, commands):
        machine_dir = "/apps/groupvte/machines/%s" % (machine) 
        self.client.set_list(machine_dir + "/commands", gconf.VALUE_STRING, commands)
        self.client.suggest_sync()

    def add_session(self, session, machinelist):
        session_dir = "/apps/groupvte/sessions/%s" % (session)
        if self.client.dir_exists(session_dir):
            return False
        
        self.client.add_dir(session_dir, gconf.CLIENT_PRELOAD_ONELEVEL)
        self.client.set_list(session_dir + "/machines", gconf.VALUE_STRING, machinelist)
        self.client.suggest_sync()
        return True
    
    def change_session(self, session, machinelist):
        session_dir = "/apps/groupvte/sessions/%s" % (session) 
        self.client.set_list(session_dir + "/machines", gconf.VALUE_STRING, machinelist)
        self.client.suggest_sync()
        
    def remove_machine(self, machine):
        machine_dir = "/apps/groupvte/machines/%s" % (machine) 
        commands    = "%s/commands" % (machine_dir)
        if self.client.dir_exists(machine_dir):
            self.client.unset(commands)
            self.client.remove_dir(machine_dir)
            self.client.suggest_sync()
    
    def remove_session(self, session):
        session_dir = "/apps/groupvte/sessions/%s" % (session)
        machines    = "%s/machines" % (session_dir)
        if self.client.dir_exists(session_dir):
            self.client.unset(machines)
            self.client.remove_dir(session_dir)
            self.client.suggest_sync()
    
    def __init__(self):
        self.client = gconf.client_get_default ()
        self.client.add_dir ("/apps/groupvte", gconf.CLIENT_PRELOAD_ONELEVEL)

class SettingsUI:
    _maxcommand    = 9
    _maxmachinelen = 40
    _maxsessionlen = 40
    _maxcmdlen     = 80
 
    def add_session_cb(self, widget, data):
        logging.debug("add session")
        machines = self.conf.get_machine_list()

        dialog = gtk.Dialog('Add session', self.window,
            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_ADD, gtk.RESPONSE_ACCEPT))
        dialog.set_resizable(False)
        dialog.set_border_width(12)
        dialog.vbox.set_spacing(6)
        hbox = gtk.HBox()
        sessionlabel= gtk.Label("Session name:")
        sessionname = gtk.Entry(self._maxsessionlen)
        hbox.pack_start(sessionlabel)
        hbox.pack_start(sessionname)
        dialog.vbox.pack_start(hbox)
        dialog.vbox.pack_start(gtk.HSeparator())

        logging.debug("Available machines %s" % machines)
        sess = []
        for i in range(len(machines)):
            sess.append((machines[i], gtk.CheckButton(machines[i])))
            dialog.vbox.pack_start(sess[i][1])
            
        dialog.show_all()
        r = dialog.run()
        dialog.destroy()
        if r == gtk.RESPONSE_ACCEPT:
            name = sessionname.get_text()
            if len(name) == 0:
                return
            chosenmachines = []
            for i in range(len(sess)):
                if sess[i][1].get_active():
                    logging.debug("Add session %s" % sess[i][0])
                    chosenmachines.append(sess[i][0])
            
            logging.debug("chosen machines %s" % chosenmachines)
            self.conf.add_session(name, chosenmachines)
            self.sessionstore.append(['%s' % name])
        
    def edit_session_cb(self, widget, data):
        logging.debug("edit session")
        selected = []
        selection_object = data.get_selection()
        selection_object.selected_foreach(lambda model, path, iter_obj, sel =
            selected: sel.append(path))
        logging.debug("sel %s" % selected)
        if len(selected) == 0:
            return
        
        iter_obj = self.sessionstore.get_iter(selected[0][0])
        session = self.sessionstore.get_value(iter_obj, 0)
        machines = self.conf.get_session(session)
        machinelist = self.conf.get_machine_list()
        
        dialog = gtk.Dialog('Edit session', self.window,
            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_ADD, gtk.RESPONSE_ACCEPT))
        dialog.set_resizable(False)
        dialog.set_border_width(12)
        dialog.vbox.set_spacing(6)
        hbox = gtk.HBox()
        sessionlabel= gtk.Label("Session name:")
        sessionname = gtk.Entry(self._maxsessionlen)
        sessionname.set_text(session)
        
        hbox.pack_start(sessionlabel)
        hbox.pack_start(sessionname)
        dialog.vbox.pack_start(hbox)
        dialog.vbox.pack_start(gtk.HSeparator())

        m = []
        for i in range(len(machinelist)):
            m.append((machinelist[i], gtk.CheckButton(machinelist[i])))
            for j in range(len(machines)):
                if machines[j] == machinelist[i]:
                    m[i][1].set_active(True)
            dialog.vbox.pack_start(m[i][1])
            
        dialog.show_all()
        r = dialog.run()
        dialog.destroy()
        if r == gtk.RESPONSE_ACCEPT:
            name = sessionname.get_text()
            if len(name) == 0:
                return

            chosenmachines = []
            for i in range(len(m)):
                if m[i][1].get_active():
                    chosenmachines.append(m[i][0])

            logging.debug("name %s machines %s" % (name, chosenmachines))
            if name == session:
                self.conf.change_session(session, chosenmachines)
            else:
                logging.debug("removing session %s" % (session))
                self.conf.remove_session(session)
                self.sessionstore.remove(iter_obj)
                logging.debug("adding session %s with machines %s" % (name, chosenmachines))
                self.conf.add_session(name, chosenmachines)
                logging.debug("Added")
                self.sessionstore.append(['%s' % name])
 
    def delete_session_cb(self, widget, data):
        selected = []
        selection_object = data.get_selection()
        selection_object.selected_foreach(lambda model, path, iter_obj, sel =
            selected: sel.append(path))
        logging.debug("sel %s" % selected)
        if len(selected) == 0:
            return

        iter_obj = self.sessionstore.get_iter(selected[0][0])
        value = self.sessionstore.get_value(iter_obj, 0)
        if value.strip().lower() == "default":
            return

        dialog = gtk.Dialog('Quit', self.window,
            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_DELETE, gtk.RESPONSE_ACCEPT))
        dialog.set_resizable(False)
        dialog.set_border_width(12)
        dialog.vbox.set_spacing(6)
        image = gtk.Image()
        image.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)
        description = gtk.Label('Are you sure you want to erase session <b>%s</b>?\n' % (value))
        description.set_use_markup(True)
        description.set_justify(gtk.JUSTIFY_FILL)
        description.set_line_wrap(True)

        dialog.vbox.pack_start(image)
        dialog.vbox.pack_start(description)

        dialog.vbox.show_all()
        r = dialog.run()
        dialog.destroy()
        if r == gtk.RESPONSE_ACCEPT:
            logging.debug("Erasing session %s" % value)
            self.conf.remove_session(value)
            logging.debug("Erase iter %s" % iter_obj)
            self.sessionstore.remove(iter_obj)


    def add_machine_cb(self, widget, data):
        logging.debug("add machine")
        dialog = gtk.Dialog('Add machine', self.window,
            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_ADD, gtk.RESPONSE_ACCEPT))
        dialog.set_resizable(False)
        dialog.set_border_width(12)
        dialog.vbox.set_spacing(6)
        hbox = gtk.HBox()
        machinelabel= gtk.Label("Machine name:")
        machinename = gtk.Entry(self._maxmachinelen)
        hbox.pack_start(machinelabel)
        hbox.pack_start(machinename)
        dialog.vbox.pack_start(hbox)
        dialog.vbox.pack_start(gtk.HSeparator())

        cmdlabel = []
        cmds     = []
        for i in range(self._maxcommand):
            cmds.append(gtk.Entry(self._maxcmdlen))
            cmdlabel.append(gtk.Label("Command nr. %d :" % (i)))
            hbox = gtk.HBox()
            hbox.set_spacing(5)
            hbox.pack_start(cmdlabel[i])
            hbox.pack_start(cmds[i])
            dialog.vbox.pack_start(hbox)
            
        dialog.show_all()
        r = dialog.run()
        dialog.destroy()
        if r == gtk.RESPONSE_ACCEPT:
            name = machinename.get_text()
            if len(name) == 0:
                return
            commands = []
            for i in range(self._maxcommand):
                s = cmds[i].get_text().strip()
                if len(s) > 0:
                    commands.append(s)

            logging.debug("adding machine %s" % name)
            logging.debug("- commands %s" % commands)
            self.conf.add_machine(name, commands)
            logging.debug("Added")
            self.machinestore.append(['%s' % name])
    
    def edit_machine_cb(self, widget, data):
        logging.debug("edit machine")
        selected = []
        selection_object = data.get_selection()
        selection_object.selected_foreach(lambda model, path, iter_obj, sel =
            selected: sel.append(path))
        logging.debug("sel %s" % selected)
        if len(selected) == 0:
            return
        
        iter_obj = self.machinestore.get_iter(selected[0][0])
        machine = self.machinestore.get_value(iter_obj, 0)
        # We don't edit the default machine (i.e. a simple local bash)
        if machine.strip().lower() == "default":
            return
        commands = self.conf.get_machine(machine)
            
        dialog = gtk.Dialog('Edit machine', self.window,
            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_ADD, gtk.RESPONSE_ACCEPT))
        dialog.set_resizable(False)
        dialog.set_border_width(12)
        dialog.vbox.set_spacing(6)
        hbox = gtk.HBox()
        machinelabel= gtk.Label("Machine name:")
        machinename = gtk.Entry(self._maxmachinelen)
        machinename.set_text(machine)
        
        hbox.pack_start(machinelabel)
        hbox.pack_start(machinename)
        dialog.vbox.pack_start(hbox)
        dialog.vbox.pack_start(gtk.HSeparator())

        cmdlabel = []
        cmds     = []
        for i in range(self._maxcommand):
            cmds.append(gtk.Entry(self._maxcmdlen))
            cmdlabel.append(gtk.Label("Command nr. %d :" % (i)))
            if i < len(commands):
                cmds[i].set_text(commands[i])
            hbox = gtk.HBox()
            hbox.set_spacing(5)
            hbox.pack_start(cmdlabel[i])
            hbox.pack_start(cmds[i])
            dialog.vbox.pack_start(hbox)
            
        dialog.show_all()
        r = dialog.run()
        dialog.destroy()
        if r == gtk.RESPONSE_ACCEPT:
            name = machinename.get_text()
            commands = []
            for i in range(self._maxcommand):
                s = cmds[i].get_text().strip()
                if len(s) > 0:
                    commands.append(s)

            logging.debug("name %s machine %s" % (name, machine))
            if name == machine:
                self.conf.change_machine(machine, commands)
            else:
                logging.debug("removing machine %s" % (machine))
                self.conf.remove_machine(machine)
                self.machinestore.remove(iter_obj)
                logging.debug("adding machine %s with commands %s" % (name, commands))
                self.conf.add_machine(name, commands)
                logging.debug("Added")
                self.machinestore.append(['%s' % name])

    def delete_machine_cb(self, widget, data):
        logging.debug("delete machine")
        selected = []
        selection_object = data.get_selection()
        selection_object.selected_foreach(lambda model, path, iter_obj, sel =
            selected: sel.append(path))
        logging.debug("sel %s" % selected)
        if len(selected) == 0:
            return

        iter_obj = self.machinestore.get_iter(selected[0][0])
        machine = self.machinestore.get_value(iter_obj, 0)
        if machine.strip().lower() == "default":
            return

        dialog = gtk.Dialog('Delete machine', self.window,
            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_DELETE, gtk.RESPONSE_ACCEPT))
        dialog.set_resizable(False)
        dialog.set_border_width(12)
        dialog.vbox.set_spacing(6)
        image = gtk.Image()
        image.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)
        description = gtk.Label('Are you sure you want to erase the <b>%s</b> machine?\n' % (machine))
        description.set_use_markup(True)
        description.set_justify(gtk.JUSTIFY_FILL)
        description.set_line_wrap(True)
        dialog.vbox.pack_start(image)
        dialog.vbox.pack_start(description)

        dialog.vbox.show_all()
        r = dialog.run()
        dialog.destroy()
        if r == gtk.RESPONSE_ACCEPT:
            logging.debug("Erasing machine %s" % machine)
            self.conf.remove_machine(machine)
            logging.debug("Erase iter %s" % iter_obj)
            self.machinestore.remove(iter_obj)
                                

    def __init__(self, window, info, conf):
        self.window = window
        self.conf   = conf
        self.dialog = gtk.Dialog('Settings', window,
            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
            (gtk.STOCK_CLOSE, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
        notebook = gtk.Notebook()

        # User Interface
        a = gtk.Label("_User interface")
        a.set_use_underline(True)
        v = gtk.VBox()
        v.set_spacing(10)
        v.set_border_width(12)
        fontlabel = gtk.Label("<b>Terminal font</b>")
        fontlabel.set_use_markup(True)
        lb = gtk.HBox()
        lb.pack_start(fontlabel, False, False, 0)
        v.pack_start(lb)
        
        self.fontselector = gtk.FontSelection()
        logging.debug("selected font %s" % self.conf.get_font())
        # FIXME: why isn't it working?
        logging.debug("could font be set %s" % self.fontselector.set_font_name(self.conf.get_font()))
        v.pack_start(self.fontselector)

        self.bell = gtk.CheckButton("Terminal bell")
        v.pack_start(self.bell)
        
        defaultviewlabel = gtk.Label("<b>Default view</b>")
        defaultviewlabel.set_use_markup(True)
        lb = gtk.HBox()
        lb.pack_start(defaultviewlabel, False, False, 0)
        v.pack_start(lb)

        v2 = gtk.VBox()
        button = gtk.RadioButton(None, "Notebook view")
        v2.pack_start(button)
        button.show()
        
        button = gtk.RadioButton(button, "Table view")
        button.set_active(True)
        v2.pack_start(button)
        button.show()
        v.pack_start(v2)
        notebook.append_page(v, a)
        
        # Sessions
        a = gtk.Label("_Sessions")
        a.set_use_underline(True)
        v = gtk.VBox()
        v.set_spacing(10)
        v.set_border_width(12)
        hb = gtk.HBox()
        hb.set_spacing(10)
        bb = gtk.VButtonBox()
        bb.set_spacing(5)
        bb.set_layout(gtk.BUTTONBOX_START)
        add   = gtk.Button("Add", gtk.STOCK_ADD)
        edit  = gtk.Button("Edit", gtk.STOCK_EDIT)
        delete= gtk.Button("Delete", gtk.STOCK_DELETE)
        bb.pack_start(add)
        bb.pack_start(edit)
        bb.pack_start(delete)
        
        self.sessionstore = gtk.ListStore(str)
        for parent in self.conf.get_session_list():
            if parent.lower() != "default":
                self.sessionstore.append(['%s' % parent])

        self.treesessionview = gtk.TreeView(self.sessionstore)
        self.tvsessioncolumn = gtk.TreeViewColumn('Sessions')
        self.treesessionview.append_column(self.tvsessioncolumn)
        self.cellsession = gtk.CellRendererText()
        self.tvsessioncolumn.pack_start(self.cellsession, True)
        self.tvsessioncolumn.add_attribute(self.cellsession, 'text', 0)
        self.treesessionview.set_search_column(0)
        self.tvsessioncolumn.set_sort_column_id(0)
        self.treesessionview.set_reorderable(True)
        add.connect("clicked", self.add_session_cb, self.treesessionview)
        edit.connect("clicked", self.edit_session_cb, self.treesessionview)
        delete.connect("clicked", self.delete_session_cb, self.treesessionview)

        hb.pack_start(self.treesessionview, True, True, 0)
        hb.pack_start(bb, False, False, 0)
        v.pack_start(hb, True, True, 0)
        
        notebook.append_page(v, a)

        # Machines
        a = gtk.Label("_Machines")
        a.set_use_underline(True)
        v = gtk.VBox()
        v.set_spacing(10)
        v.set_border_width(12)
        hb = gtk.HBox()
        hb.set_spacing(10)
        bb = gtk.VButtonBox()
        bb.set_spacing(5)
        bb.set_layout(gtk.BUTTONBOX_START)
        add   = gtk.Button("Add", gtk.STOCK_ADD)
        edit  = gtk.Button("Edit", gtk.STOCK_EDIT)
        delete= gtk.Button("Delete", gtk.STOCK_DELETE)
        bb.pack_start(add)
        bb.pack_start(edit)
        bb.pack_start(delete)
        
        self.machinestore = gtk.ListStore(str)
        for parent in self.conf.get_machine_list():
            if parent.lower() != "default":
                self.machinestore.append(['%s' % parent])

        self.treemachineview = gtk.TreeView(self.machinestore)
        self.tvmachinecolumn = gtk.TreeViewColumn('Machines')
        self.treemachineview.append_column(self.tvmachinecolumn)
        self.cellmachine = gtk.CellRendererText()
        self.tvmachinecolumn.pack_start(self.cellmachine, True)
        self.tvmachinecolumn.add_attribute(self.cellmachine, 'text', 0)
        self.treemachineview.set_search_column(0)
        self.tvmachinecolumn.set_sort_column_id(0)
        self.treemachineview.set_reorderable(True)
        hb.pack_start(self.treemachineview, True, True, 0)
        hb.pack_start(bb, False, False, 0)
        v.pack_start(hb, True, True, 0)
        
        add.connect("clicked", self.add_machine_cb, self.treemachineview)
        edit.connect("clicked", self.edit_machine_cb, self.treemachineview)
        delete.connect("clicked", self.delete_machine_cb, self.treemachineview)
        
        notebook.append_page(v, a)

        self.dialog.vbox.pack_start(notebook)

        self.dialog.vbox.show_all()
        ret = self.dialog.run()
        self.dialog.destroy()
        if ret == gtk.RESPONSE_ACCEPT:
            logging.debug("Selected font %s" % self.fontselector.get_font_name())
            logging.debug("Terminal bell %s" % self.bell.get_active())
