# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# groupvte - vteview
# Copyright (C) 2005 Michele Baldessari <michele@pupazzo.org>

# This file may be distributed and/or modified under the terms of
# the GNU General Public License version 2 as published by
# the Free Software Foundation.
# This file is distributed without any warranty; without even the implied
# warranty of merchantability or fitness for a particular purpose.
# See "LICENSE.GPL" in the source distribution for more information.

import gtk
import gtk.gdk
import logging

try:
    from _vte import *
except:
    logging.warning("_vte python module not found..trying vte")
    try:
        from vte import *
        logging.info("Successfully imported vte")
    except:
        logging.error("No vte nor _vte found, exiting")
        sys.exit(-1)


def print_all(s, widget):
    p = widget.get_parent()
    logging.debug("%s %s (parent is %s)" % (s, widget, p))
    try:
        c = widget.get_children()
        for i in c:
            t = i.get_parent()
            logging.debug("  Child %s has parent %s" % (i, t))
    except AttributeError:
        pass


class VteStruct:
    def __init__(self):
		# Terminal defaults.
		self.audible = 0
		self.background = None
		self.blink = 0
		self.command = None
		self.emulation = "xterm"
		self.font = "fixed"
		self.scrollback = 1000
		self.transparent = 0
		self.visible = 0
        self.term = Terminal()

		if (self.background):
			self.term.set_background_image(self.background)
		if (self.transparent):
			self.term.set_background_transparent(True)

		self.term.set_cursor_blinks(self.blink)
		self.term.set_emulation(self.emulation)
		self.term.set_font_from_string(self.font)
		self.term.set_scrollback_lines(self.scrollback)
		self.term.set_audible_bell(self.audible)
		self.term.set_visible_bell(self.visible)

		self.scrollbar = gtk.VScrollbar()
		self.hbox      = gtk.HBox()
		self.child_pid = self.term.fork_command()
        self.label     = gtk.Label()

class VteView:
    VIEW_NOTEBOOK = 1
    VIEW_TABLE    = 2
    
    # terminals is a list of vtestruct
    def __init__(self, viewtype=VIEW_TABLE, terminals=[]):
        self.currentterminal = 0
        self.totalterminals  = 0
        self.currentview     = viewtype
        if (viewtype == self.VIEW_NOTEBOOK):
            self.view = gtk.Notebook()
            self.view.popup_enable()
            self.view.connect("switch-page", self.switch_page_cb)
        elif (viewtype == self.VIEW_TABLE):
            self.view = gtk.Table(1, 1, True)

        if (len(terminals) > 0):
            for i in terminals:
                self.append_tab(i)

	def switch_page_cb(self, page, page_num, tab_number):
        self.set_label_color(tab_number, "black")
        return
    
    def set_label_color(self, tab, newcolor, bold=False):
        if self.currentview == self.VIEW_NOTEBOOK:
            w = self.view.get_nth_page(tab)
            label = self.view.get_tab_label(w)
            label.set_use_markup(True)
        elif self.currentview == self.VIEW_TABLE:
            tabs = self.view.get_children()
            w = tabs[tab]
            label = w.get_children()[0]
            label.set_use_markup(True)

        if bold:
            label.set_markup("<span foreground='%s'><b>%s</b></span>" % \
                (newcolor, label.get_text()))
        else:
            label.set_markup("<span foreground='%s'>%s</span>" % \
                (newcolor, label.get_text()))

    def remove_tab(self, tab):
        if self.currentview == self.VIEW_NOTEBOOK:
            self.view.remove_page(tab)
            self.totalterminals = self.totalterminals - 1
        elif self.currentview == self.VIEW_TABLE:
            tabs = self.view.get_children()
            w = tabs[tab]
            self.view.remove(w)
            self.totalterminals = self.totalterminals - 1

    def remove_all_tabs(self):
        if self.currentview == self.VIEW_NOTEBOOK:
            for i in range(self.view.get_n_pages()):
                self.view.remove_page(-1)
        elif self.currentview == self.VIEW_TABLE:
            tabs = self.view.get_children()
            for i in tabs:
                self.view.remove(i)

        self.totalterminals = 0

    def append_tab(self, vtestruct, size=None):
        if self.currentview == self.VIEW_NOTEBOOK:
            vtestruct.label.unparent()
            vtestruct.hbox.unparent()
            self.view.append_page(vtestruct.hbox, vtestruct.label)
            self.view.show_all()
        elif self.currentview == self.VIEW_TABLE:
            # We try to grow to the right, if the space ends we grow downwards 
            # FIXME: this is broken for > 4 tabs..what's a right solution? viewports or
            # at least checking the available screen space and then decide
            logging.debug("current window_size %s" % size)
            termsize = vtestruct.hbox.size_request()
            logging.debug("about to add %s" % termsize)
            box = gtk.VBox()
            box.pack_start(vtestruct.label, False, False)
            box.pack_start(vtestruct.hbox, True, True)
            if self.totalterminals < 2:
                self.view.resize(1, self.totalterminals + 1)
                self.view.attach(box, self.totalterminals, self.totalterminals + 1, 0, 1)
            else: 
                t = (self.totalterminals) % 2
                d = self.totalterminals / 2
                self.view.resize(2, t + 1)
                self.view.attach(box, t, t + 1, d, d + 1)
            self.view.show_all()
            
        self.totalterminals = self.totalterminals + 1

    def get_current_tab(self):
        if self.currentview == self.VIEW_NOTEBOOK:
            return self.view.get_current_page()
        elif self.currentview == self.VIEW_TABLE:
            c = self.view.get_children()
            counter = 0
            for i in c:
                c = i.get_children()
                vt = c[1]
                if vt.is_focus():
                    return counter
                counter = counter + 1
            
            return -1
        else:
            return -1

    def set_current_tab(self, tab):
        if self.currentview == self.VIEW_NOTEBOOK:
            self.currentterminal = tab
            self.view.set_current_page(tab)
        elif self.currentview == self.VIEW_TABLE:
            children = self.view.get_children()
            for i in children:
                l = i.get_children()[0]
                l.set_markup("<span foreground='%s'>%s</span>" % ("black", l.get_text()))

    def destroy_view(self):
        self.view.destroy()
        del self.view

    def get_view(self):
        return self.view
